import ipdb
import numpy as np
import matplotlib.pyplot as plt
from joblib import dump, load

from sklearn.cluster import KMeans
from sklearn.model_selection import GridSearchCV
from sklearn.datasets import make_classification

from config import MIN_RESP_RATE, MAX_RESP_RATE

TUNER_TYPE = "gridsearchcv"
DEBUG = False

class KMeansClass():
    ''' Class to handle the linear discriminant analysis under gridsearch, must
    have unilateral data classification'''
    def __init__(self):
        # model parameters
        if DEBUG:
            self.n_clusters = 3
            self.param_grid = [
                {
                    "init": ["k-means++"],
                    "tol": [1e-3],
                    "n_init": [10, 30],
                    "max_iter": [100, 300],
                },
            ]
            self.verbose = 1
        else:
            self.n_clusters = MAX_RESP_RATE - MIN_RESP_RATE
            self.param_grid = [
                {
                    "init": ["k-means++","random"],
                    "tol": [1e-3, 3e-3, 1e-4, 3e-4],
                    "n_init": [10, 30, 100],
                    "max_iter": [100, 300, 1000, 3000],
                },
            ]
            self.verbose = 0
        self.scoring = ['neg_mean_absolute_error',
                        'neg_root_mean_squared_error',
                        'normalized_mutual_info_score',]

        self.eval_csv = './evaluations/knn.csv'

    def build(self):
        self.model = GridSearchCV(
            KMeans(n_clusters=self.n_clusters, verbose=self.verbose),
            param_grid=self.param_grid,
            scoring=self.scoring,
            cv=3,
            refit='normalized_mutual_info_score'
        )
        return self.model

    def save_model(self, filename='kmeans_model.joblib'):
        # save the model to disk
        with open(filename, 'wb') as mfile:
            dump(self.model, mfile)

    def load_model(self, filename='kmeans_model.joblib'):
        # load the model from disk
        # self.model = load(filename)
        with open(filename, 'rb') as mfile:
            self.model = load(mfile)

def main():
    # test classification dataset
    print("Making data")
    n_features = 10
    x, y = make_classification(n_samples=1000, n_features=n_features,
                               n_informative=6, n_redundant=4, random_state=1,
                               n_classes=3)
    print('classes: ', np.unique(y))

    # define model
    knn = KNNClass()
    model = knn.build()
    

    # fit model
    print("Fitting data")
    model.fit(x, y)

    # new data
    row = np.random.rand(1, n_features) + np.random.randint(low=-4, high=4,
                                                            size=(1,n_features)
                                                           )
    print("row: ", row)
    print("Predicting")
    yhat = model.predict(row)

    print('Predicted Class: %d' % yhat)

    plt.plot(x[y==0,0], x[y==0,1], 'b.')
    plt.plot(x[y==1,0], x[y==1,1], 'r.')
    plt.plot(x[y==2,0], x[y==2,1], 'k.')
    plt.plot(row[0,0], row[0, 1], 'y^')
    plt.show()

if __name__ == '__main__':
    main()
