import ipdb
import numpy as np
import matplotlib.pyplot as plt
from joblib import dump, load

from sklearn.svm import LinearSVC
from sklearn.model_selection import GridSearchCV
from sklearn.multiclass import OneVsRestClassifier
from sklearn.multioutput import MultiOutputClassifier
from sklearn.datasets import make_classification

from config import MIN_RESP_RATE, MAX_RESP_RATE

MODEL_TYPE = "svm"
TUNER_TYPE = "gridsearchcv"
SVM_DEBUG = True

class RandomPredictor():
    ''' Class to handle the linear discriminant analysis under gridsearch, must
    have unilateral data classification'''
    def __init__(self, seed=12345):
        self.rng = np.random.default_rng(seed)
        self.eval_csv = './evaluations/rng.csv'
        self.min_resp_rate = MIN_RESP_RATE
        self.max_resp_rate = MAX_RESP_RATE

    def predict(self, size):
        rint = self.rng.integers(low=self.min_resp_rate,
                                 high=self.max_resp_rate,
                                 size=size)
        return rint

    def save_model(self, filename='random.joblib'):
        # save the model to disk
        with open(filename, 'wb') as mfile:
            dump(self.model, mfile)

    def load_model(self, filename='random.joblib'):
        # load the model from disk
        # self.model = load(filename)
        with open(filename, 'rb') as mfile:
            self.model = load(mfile)

def main():
    # test classification dataset
    print("Making data")
    n_features = 10
    x, y = make_classification(n_samples=1000, n_features=n_features,
                               n_informative=6, n_redundant=4, random_state=1,
                               n_classes=3)
    print('classes: ', np.unique(y))

    # define model
    model = RandomPredictor()

    # fit model
    print("Fitting data")
    model.fit(x, y)

    # new data
    row = np.random.rand(1, n_features) + np.random.randint(low=-4, high=4,
                                                            size=(1,n_features)
                                                           )
    print("row: ", row)
    print("Predicting")
    yhat = model.predict(row)

    print('Predicted Class: %d' % yhat)

    plt.plot(x[y==0,0], x[y==0,1], 'b.')
    plt.plot(x[y==1,0], x[y==1,1], 'r.')
    plt.plot(x[y==2,0], x[y==2,1], 'k.')
    plt.plot(row[0,0], row[0, 1], 'y^')
    plt.show()

if __name__ == '__main__':
    main()
